#ifndef SLOW_COPY_HPP
#define SLOW_COPY_HPP

#include <chrono>
#include <thread>

class SlowCopyBase
{
public:
    virtual void operator()()
    {
    }
};

template<typename Dst, typename Src>
class SlowCopy : public SlowCopyBase
{
public:
    SlowCopy(Dst& dst, Src& src)
    : dst_(dst)
    , src_(src)
    {
    }

    void operator()()
    {
        while (1)
        {
            _wait();

            char character = src_();
            
            if (character == 'x') break;
            
            dst_(character);
        }
    }

private: 
    void _wait()
    {
        int msWait = 100;
        std::this_thread::sleep_for(std::chrono::milliseconds(msWait));
    }

    Dst& dst_;
    Src& src_;
};

#if __cplusplus >= 201703L //c++17
template<typename Src, typename Dst>
SlowCopy(Src&, Dst&) -> SlowCopy<Src, Dst>;
#endif

#endif // SLOW_COPY_HPP
